<?xml version='1.0' encoding="UTF-8" ?>
<!--
****************************************************************
DITA to XSL-FO Stylesheet
Module: Merged file conversion templates
Copyright © 2009-2012 Antenna House, Inc. All rights reserved.
Antenna House is a trademark of Antenna House, Inc.
URL    : http://www.antennahouse.com/
E-mail : info@antennahouse.com
****************************************************************
-->
<xsl:stylesheet version="2.0" 
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:xs="http://www.w3.org/2001/XMLSchema"
 xmlns:ahf="http://www.antennahouse.com/names/XSLT/Functions/Document"
 exclude-result-prefixes="xs ahf">

<xsl:output method="xml" encoding="UTF-8"/>

<!-- 
 function:	General template for all element
 param:		none
 return:	copied result
 note:		
 -->
<xsl:template match="*">
    <xsl:copy>
        <xsl:copy-of select="@*"/>
        <xsl:apply-templates/>
    </xsl:copy>
</xsl:template>

<!--
 function:	topicref
 param:		none
 return:	self and descendant element or none
 note:		if @print="no", ignore it.
 -->
<xsl:template match="*[contains(@class,' map/topicref ')]">
    <xsl:variable name="isIgnorebleTopicref" select="ahf:isIgnorebleTopicref(.)" as="xs:boolean"/>
	<xsl:choose>
		<xsl:when test="@print='no'" />
	    <xsl:when test="$isIgnorebleTopicref">
	        <xsl:apply-templates/>
	    </xsl:when>
        <xsl:otherwise>
            <xsl:copy>
                <xsl:copy-of select="@*"/>
                <xsl:apply-templates/>
            </xsl:copy>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>

<!-- 
 function:	comment template
 param:		none
 return:	comment 
 note:		none
 -->
<xsl:template match="comment()">
    <xsl:copy/>
</xsl:template>

<!-- 
 function:	processing-instruction template
 param:		nome
 return:	processing-instruction
 note:		
 -->
<xsl:template match="processing-instruction()">
    <xsl:copy/>
</xsl:template>

<!-- 
 function:	Judge the parameter topicref is ignoreble
 param:		prmTopicRef
 return:	xs:boolean
 note:		<topicgroup> must be removed unconditionally.
 -->
<xsl:function name="ahf:isIgnorebleTopicref">
    <xsl:param name="prmTopicRef" as="element()"/>
    
    <xsl:variable name="hasNavtitle" as="xs:boolean" select="string($prmTopicRef/*[contains(@class, ' map/topicmeta ')]/*[contains(@class, ' topic/navtitle ')]) or exists($prmTopicRef/@navtitle) "/>
    <xsl:choose>
        <xsl:when test="$prmTopicRef[contains(@class,' mapgroup-d/topicgroup ')]">
            <xsl:sequence select="true()"/>
        </xsl:when>
        <xsl:when test="exists($prmTopicRef/@href)">
            <xsl:sequence select="false()"/>
        </xsl:when>
        <xsl:when test="$hasNavtitle">
            <xsl:sequence select="false()"/>
        </xsl:when>
        <xsl:when test="$prmTopicRef[string(@class) eq '- map/topicref']">
            <xsl:sequence select="true()"/>
        </xsl:when>
        <xsl:when test="$prmTopicRef[contains(@class,' mapgroup-d/topicset ')]">
            <xsl:sequence select="true()"/>
        </xsl:when>
        <xsl:when test="$prmTopicRef[contains(@class,' mapgroup-d/topicsetref ')]">
            <xsl:sequence select="true()"/>
        </xsl:when>
        <xsl:when test="$prmTopicRef[contains(@class,' mapgroup-d/topichead ')]">
            <xsl:sequence select="true()"/>
        </xsl:when>
        <xsl:when test="$prmTopicRef[contains(@class,' map/anchor ')]">
            <xsl:sequence select="true()"/>
        </xsl:when>
        <xsl:otherwise>
            <!-- Not ordinal 'topicref' element -->
            <xsl:sequence select="false()"/>
        </xsl:otherwise>
    </xsl:choose>
</xsl:function>

</xsl:stylesheet>
